########################################################################
# SwarmOps - Heuristic optimization for Python.
# Copyright (C) 2003-2016 Magnus Erik Hvass Pedersen.
# See the file README.md for instructions.
# See the file LICENSE.txt for license details.
# SwarmOps on the internet: http://www.Hvass-Labs.org/
########################################################################

########################################################################
# Parent-classes for doing optimization. These provide various logistics,
# printing of status messages, running multiple optimizations, etc.
########################################################################

import numpy as np
from swarmops.FitnessTrace import FitnessTrace
from swarmops import tools


##################################################

class SingleRun:
    """
    Parent-class for performing a single optimization run.
    The class provides various logistics that are common to all
    optimizers. If you make a new optimizer, then you should derive
    from this class.
    """

    def __init__(self, max_evaluations, run_number=0, trace_len=0, display_interval=0):
        """
        Create object instance and initialize variables for keeping track
        of the optimization progress.
        Then call self._optimize() to perform the actual optimization.

        :param max_evaluations:
            Maximum number of fitness evaluations for the problem.

        :param run_number:
            The optimization run number, if performing multiple optimization runs.

        :param trace_len:
            Approximate length of the fitness-trace.
            Default is zero which means that no fitness-trace will be created.

        :param display_interval:
            Approximate interval between printing status messages.
            Default is zero which means that no status messages will be printed.

        :return:
            Object instance.
        """

        # Copy arguments to instance variables.
        self.max_evaluations = max_evaluations
        self.run_number = run_number
        self.display_interval = display_interval

        # Initialize the counter for the next status-display.
        self.display_next = 0

        # Create an object used for tracing the fitness at regular intervals.
        self.fitness_trace = FitnessTrace(trace_len, max_evaluations)

        # Initialize best-known position and fitness.
        self.best = None
        self.best_fitness = np.inf

        # Print status at the beginning of the optimization run.
        self._start_run()

        # Perform the actual optimization iterations.
        self._optimize()

        # Print status at the end of the optimization run.
        self._end_run()

    def _optimize(self):
        """
        Perform the actual optimization.
        This function should be implemented by the child-class.
        The function does not return anything, instead it will
        call self._update_best() to update the best-known solution
        during optimization.
        """

        # Raise an exception if the child-class has not implemented this function.
        raise NotImplementedError

    def _start_run(self):
        """
        Print status at the beginning of the optimization run.
        """

        if self.display_interval > 0:
            msg = "Starting optimization run {0} using {1} ..."
            print(msg.format(self.run_number, self.name))

    def _iteration(self, i):
        """
        Print status and trace fitness during optimization.
        This function should be called regularly during optimization.
        """

        if self.display_interval > 0 and i >= self.display_next:
            # Print the status.
            msg = "Run: {0}, Iteration: {1}, Best Fitness: {2:.4e}"
            print(msg.format(self.run_number, i, self.best_fitness))

            # Increment the counter for the next status-display.
            self.display_next = i + self.display_interval

        # Trace the fitness.
        self.fitness_trace.trace(i, self.best_fitness)

    def _end_run(self):
        """
        Print status at the end of the optimization run.
        """

        if self.display_interval > 0:
            # Print the status.
            msg = "Finished optimization run {0}, Best fitness: {1:.4e}"
            print(msg.format(self.run_number, self.best_fitness))

    def _update_best(self, fitness, x):
        """
        Update the best-known solution and fitness if an improvement.

        WARNING: This function does NOT copy the array x so you must
        ensure that the optimizer does not modify the array you pass
        in as x here.

        :param fitness: New fitness.
        :param x: New solution.
        :return: Boolean whether the fitness was an improvement.
        """

        # If the fitness is an improvement over the best-known fitness.
        improvement = fitness < self.best_fitness
        if improvement:
            # Update the best-known fitness and position.
            self.best_fitness = fitness
            self.best = x

        return improvement


##################################################

class MultiRun:
    """
    Perform multiple optimization runs with an optimizer and
    calculate statistics on the results.

    This has been separated into its own class for the sake of
    modularity so it is easier to read and maintain. There is only a
    tiny overhead of instantiating the SingleRun-class for each run.
    """

    def __init__(self, optimizer, num_runs, problem, parallel=True, *args, **kwargs):
        """
        Create object instance and perform multiple optimization runs.
        To retrieve the results, access the object variables afterwards.

        The parameters are the same as for the Optimize-class,
        except for the following:

        :param optimizer:
            Class for the optimizer e.g. PSO or DE.

        :param num_runs:
            Number of optimization runs to perform.

        :param problem:
            The problem to be optimized. Instance of Problem-class.

        :param parallel:
            Perform the optimization runs in parallel (True) or serial (False).

        :param args:
            Arguments to pass along to the Optimize-class.

        :param kwargs:
            Arguments to pass along to the Optimize-class.

        :return:
            Object instance. Get the optimization results from the object's variables.
            -   best_solution is the best-found solution.
            -   best_fitness is the associated fitness of the best-found solution.
            -   fitness_trace is a 2-d numpy array with the fitness trace of each run.
        """

        # Copy arguments to instance variables.
        self.problem = problem
        self.optimizer = optimizer

        # Store the args and kwargs to be passed on to the optimizer.
        self.args = args
        self.kwargs = kwargs

        if not parallel:
            # Run the optimizer multiple times on one processor.
            self.runs = [self._optimize(run_number=i) for i in range(num_runs)]
        else:
            import multiprocessing as mp

            # Create a pool of workers sized according to the CPU cores available.
            pool = mp.Pool()

            # Run the optimizer multiple times in parallel.
            # We must use a helper-function for this.
            self.runs = pool.map(self._optimize_parallel, range(num_runs))

            # Close the pool of workers and wait for them all to finish.
            pool.close()
            pool.join()

        # Put the best solutions from all the optimization runs into an array.
        self.solution = np.array([run.best for run in self.runs])

        # Put the best fitness from all the optimization runs into an array.
        self.fitness = np.array([run.best_fitness for run in self.runs])

        # Put the fitness traces from all the optimization runs into an array.
        # This only works when the fitness-traces are all the same length.
        # If you make changes to the optimizers so they can stop early, then
        # you may have to change this.
        self.fitness_trace = np.array([run.fitness_trace.fitness for run in self.runs])

        # Index for the best fitness (minimization).
        i = np.argmin(self.fitness)

        # Best optimization run. This is an instance of the SingleRun-class.
        self.best_run = self.runs[i]

        # Best fitness of all the optimization runs.
        self.best_fitness = self.fitness[i]

        # Best solution of all the optimization runs.
        self.best = self.solution[i]

    def _optimize(self, run_number):
        """
        Helper-function used for execution of an optimization run. Non-parallel.

        :param run_number: Counter for the optimization run.
        :return: Instance of the SingleRun-class.
        """

        return self.optimizer(problem=self.problem, run_number=run_number,
                              *self.args, **self.kwargs)

    def _optimize_parallel(self, run_number):
        """
        Helper-function used for parallel execution of an optimization run.

        :param run_number: Counter for the optimization run.
        :return: Instance of the SingleRun-class.
        """

        # Create a new Pseudo-Random Number Generator for the thread.
        tools.new_prng()

        # Do the optimization.
        return self._optimize(run_number=run_number)

    def refine(self):
        """
        Refine the best result from heuristic optimization using SciPy's L-BFGS-B method.
        This may significantly improve the results on some optimization problems,
        but it is sometimes very slow to execute.

        NOTE: This function imports SciPy, which should make it possible
        to use the rest of this source-code library even if SciPy is not installed.
        SciPy should first be loaded when calling this function.

        :return:
            A tuple with:
            -   The best fitness found.
            -   The best solution found.
        """

        # SciPy requires bounds in another format.
        bounds = list(zip(self.problem.lower_bound, self.problem.upper_bound))

        # Start SciPy optimization at best found solution.
        import scipy.optimize
        res = scipy.optimize.minimize(fun=self.problem.fitness,
                                      x0=self.best,
                                      method="L-BFGS-B",
                                      bounds=bounds)

        # Get best fitness and parameters.
        refined_fitness = res.fun
        refined_solution = res.x

        return refined_fitness, refined_solution

    def print_statistics(self):
        """
        Print statistics for the fitness.
        """

        # Print header.
        print("{0} - Optimized by {1}".format(self.problem.name_full, self.optimizer.name))
        print("Fitness Statistics:")

        # Print mean and standard deviation.
        print("- Mean:\t\t{0:.4e}".format(self.fitness.mean()))
        print("- Std.:\t\t{0:.4e}".format(self.fitness.std()))

        # Print quartiles.
        print("- Min:\t\t{0:.4e}".format(self.fitness.min()))
        print("- 1st Qrt.:\t{0:.4e}".format(np.percentile(self.fitness, 0.25)))
        print("- Median:\t{0:.4e}".format(np.percentile(self.fitness, 0.5)))
        print("- 3rd Qrt.:\t{0:.4e}".format(np.percentile(self.fitness, 0.75)))
        print("- Max:\t\t{0:.4e}".format(self.fitness.max()))

    def plot_fitness_trace(self, y_log_scale=True, filename=None):
        """
        Plot the fitness traces.

        NOTE: This function imports matplotlib, which should make it possible
        to use the rest of this source-code library even if it is not installed.
        matplotlib should first be loaded when calling this function.

        :param y_log_scale: Use log-scale for y-axis.
        :param filename: Output filename e.g. "foo.svg". If None then plot to screen.
        :return: Nothing.
        """

        import matplotlib.pyplot as plt

        # Setup plotting.
        plt.grid()

        # Axis labels.
        plt.xlabel("Iteration")
        plt.ylabel("Fitness (Lower is better)")

        # Title.
        title = "{0} - Optimized by {1}".format(self.problem.name_full, self.optimizer.name)
        plt.title(title)

        # Use log-scale for Y-axis.
        if y_log_scale:
            plt.yscale("log", nonposy="clip")

        # Plot the fitness-trace for each optimization run.
        for run in self.runs:
            # Array with iteration counter for the optimization run.
            iteration = run.fitness_trace.iteration

            # Array with fitness-trace for the optimization run.
            fitness_trace = run.fitness_trace.fitness

            # Plot the fitness-trace.
            plt.plot(iteration, fitness_trace, 'r-', color='black', alpha=0.25)

        # Plot to screen or file.
        if filename is None:
            # Plot to screen.
            plt.show()
        else:
            # Plot to file.
            plt.savefig(filename, bbox_inches='tight')
            plt.close()


##################################################



########################################################################
# SwarmOps - Heuristic optimization for Python.
# Copyright (C) 2003-2016 Magnus Erik Hvass Pedersen.
# See the file README.md for instructions.
# See the file LICENSE.txt for license details.
# SwarmOps on the internet: http://www.Hvass-Labs.org/
########################################################################

########################################################################
# Differential Evolution (DE).
#
# DE is a heuristic optimizer which does not use the gradient of the
# problem. The variant implemented here is known as DE/Rand/1/Bin.
#
# The DE has some control parameters that may greatly affect its
# performance. If the default parameters do not give satisfactory
# results then you may try some of the parameters listed below
# which were tuned for different optimization scenarios, otherwise
# you can try tuning the parameters using meta-optimization.
#
# References:
#
# [1] R. Storn and K. Price. Differential evolution - a simple
#     and efficient heuristic for global optimization over
#     continuous spaces. Journal of Global Optimization,
#     11:341-359, 1997.
#
# [2] M.E.H. Pedersen. Good parameters for Differential Evolution.
#     Hvass Laboratories Report HL-1002, 2010.
#     http://www.hvass-labs.org/people/magnus/publications/pedersen10good-de.pdf
#
# ########################################################################

import numpy as np
from swarmops.Optimize import SingleRun
from swarmops import tools


##################################################

class DE(SingleRun):
    """
        Perform a single optimization run using Differential Evolution (DE).

        This is the DE/Rand/1/Bin variant.

        In practice, you would typically perform multiple optimization runs using
        the MultiRun-class. The reason is that DE is a heuristic optimizer so
        there is no guarantee that an acceptable solution is found in any single
        run. It is more likely that an acceptable solution is found if you perform
        multiple optimization runs.

        Control parameters have been tuned for different optimization scenarios.
        First try and use the default parameters. If that does not give
        satisfactory results, then you may try some of the following.
        Select the parameters that most closely match your problem.
        For example, if you want to optimize a problem where the search-space
        has 15 dimensions and you can perform 30000 evaluations, then you could
        first try using parameters_20dim_40000eval. If that does not give
        satisfactory results then you could try using parameters_10dim_20000eval.
        If that does not work then you will either need to meta-optimize the
        parameters for the problem at hand, or you should try using another optimizer.
    """

    # Name of this optimizer.
    name = "DE"
    name_full = "Differential Evolution (DE/Rand/1/Bin)"

    # Number of control parameters for DE. Used by MetaFitness-class.
    num_parameters = 3

    # Lower boundaries for the control parameters of DE. Used by MetaFitness-class.
    parameters_lower_bound = [4.0, 0.0, 0.0]

    # Upper boundaries for the control parameters of DE. Used by MetaFitness-class.
    parameters_upper_bound = [200.0, 1.0, 2.0]

    @staticmethod
    def parameters_list(num_agents, crossover_probability, differential_weight):
        """
        Create a list with DE parameters in the correct order.

        :param num_agents:
            Number of agents in the population (aka. NP in the research literature).

        :param crossover_probability:
            Crossover probability (aka. CR in the research literature).

        :param differential_weight:
            Differential weight (aka. F in the research literature).

        :return:
            List with parameters in correct order.
        """

        return [num_agents, crossover_probability, differential_weight]

    @staticmethod
    def parameters_dict(parameters):
        """
        Create and return a dict from a list of DE parameters.
        This is useful for printing the named parameters.

        :param parameters: List of parameters.
        :return: Dict with parameters.
        """

        return {'num_agents': parameters[0],
                'crossover_probability': parameters[1],
                'differential_weight': parameters[2]}

    # Default parameters for the DE which will be used if no other parameters are specified.
    # These are a compromise of the tuned parameters below. Try this first and see if it works.
    parameters_default = [40.0, 0.7, 0.7]

    # Parameters tuned by hand. These are common in the older research literature on DE.
    parameters_hand_tuned = [300.0, 0.9000, 0.5000]

    # Parameters tuned for benchmark problems in 2 dimensions using 400 fitness evaluations.
    parameters_2dim_400eval_a = [13.0, 0.7450, 0.9096]

    # Parameters tuned for benchmark problems in 2 dimensions using 400 fitness evaluations.
    parameters_2dim_400eval_b = [10.0, 0.4862, 1.1922]

    # Parameters tuned for benchmark problems in 2 dimensions using 4000 fitness evaluations.
    parameters_2dim_4000eval_a = [24.0, 0.2515, 0.8905]

    # Parameters tuned for benchmark problems in 2 dimensions using 4000 fitness evaluations.
    parameters_2dim_4000eval_b = [20.0, 0.7455, 0.9362]

    # Parameters tuned for benchmark problems in 5 dimensions using 1000 fitness evaluations.
    parameters_5dim_1000eval = [17.0, 0.7122, 0.6301]

    # Parameters tuned for benchmark problems in 5 dimensions using 10000 fitness evaluations.
    parameters_5dim_10000eval = [20.0, 0.6938, 0.9314]

    # Parameters tuned for benchmark problems in 10 dimensions using 2000 fitness evaluations.
    parameters_10dim_2000eval_a = [28.0, 0.9426, 0.6607]

    # Parameters tuned for benchmark problems in 10 dimensions using 2000 fitness evaluations.
    parameters_10dim_2000eval_b = [12.0, 0.2368, 0.6702]

    # Parameters tuned for benchmark problems in 10 dimensions using 20000 fitness evaluations.
    parameters_10dim_20000eval = [18.0, 0.5026, 0.6714]

    # Parameters tuned for benchmark problems in 20 dimensions using 40000 fitness evaluations.
    parameters_20dim_40000eval = [37.0, 0.9455, 0.6497]

    # Parameters tuned for benchmark problems in 20 dimensions using 400000 fitness evaluations.
    parameters_20dim_400000eval = [35.0, 0.4147, 0.5983]

    # Parameters tuned for benchmark problems in 30 dimensions using 6000 fitness evaluations.
    parameters_30dim_6000eval = [11.0, 0.0877, 0.6419]

    # Parameters tuned for benchmark problems in 30 dimensions using 60000 fitness evaluations.
    parameters_30dim_60000eval = [19.0, 0.1220, 0.4983]

    # Parameters tuned for benchmark problems in 50 dimensions using 100000 fitness evaluations.
    parameters_50dim_100000eval = [48.0, 0.9784, 0.6876]

    # Parameters tuned for benchmark problems in 100 dimensions using 200000 fitness evaluations.
    parameters_100dim_200000eval = [46.0, 0.9565, 0.5824]

    def __init__(self, problem, parameters=parameters_default, parallel=False, *args, **kwargs):
        """
        Create object instance and perform a single optimization run using DE.

        :param problem: The problem to be optimized. Instance of Problem-class.

        :param parameters:
            Control parameters for the DE.
            These may have a significant impact on the optimization performance.
            First try and use the default parameters and if they don't give satisfactory
            results, then experiment with other parameters or meta-optimization.

        :param parallel:
            Evaluate the fitness for the agents in parallel.
            See the README.md file for a discussion on this.

        :return:
            Object instance. Get the optimization results from the object's variables.
            -   best is the best-found solution.
            -   best_fitness is the associated fitness of the best-found solution.
            -   fitness_trace is an instance of the FitnessTrace-class.
        """

        # Copy arguments to instance variables.
        self.problem = problem
        self.parallel = parallel

        # Unpack control parameters.
        self.num_agents, self.crossover_probability, self.differential_weight = parameters

        # The number of agents must be an integer.
        self.num_agents = int(self.num_agents)

        # Initialize population with random positions in the search-space.
        # The first index is for the agent number.
        # The second index is for the search-space dimension.
        # These are actually the best-known positions for each agent
        # and will only get replaced when improvements are found.
        self.population = tools.rand_population(lower=problem.lower_init,
                                                upper=problem.upper_init,
                                                num_agents=self.num_agents,
                                                dim=problem.dim)

        # Population used for generating new agents.
        self.new_population = np.copy(self.population)

        # Initialize fitness for all agents to infinity.
        self.fitness = np.repeat(np.inf, self.num_agents)

        # Initialize parent-class which also starts the optimization run.
        SingleRun.__init__(self, *args, **kwargs)

    def _optimize(self):
        """
        Perform a single optimization run.
        This function is called by the parent-class.
        """

        # Calculate fitness for the initial agent positions.
        self._update_fitness()

        # Optimization iterations.
        # The counting starts with num_agents because the fitness has
        # already been calculated once for each agent during initialization.
        for i in range(self.num_agents, self.max_evaluations, self.num_agents):
            # Calculate the new agent positions but don't update the old yet.
            self._new_agents()

            # Calculate the fitness for each new agent position and
            # update the population if the fitness is an improvement.
            self._update_fitness()

            # Call parent-class to print status etc. during optimization.
            self._iteration(i)

    def _new_agents(self):
        """
        Calculate a new population of agents using the DE method.
        The fitness is not calculated in this function.
        """

        # Convenience variables.
        population = self.population    # 2-d array with the population of agents.
        num_agents = self.num_agents    # Number of agents in population.
        dim = self.problem.dim          # Search-space dimensionality.
        differential_weight = self.differential_weight          # Control parameter for DE.
        crossover_probability = self.crossover_probability      # Control parameter for DE.

        # Create a new population of agents.
        for i in range(num_agents):
            # This loop only processes a single agent in each iteration.
            # It might be possible to make a faster implementation using Numpy
            # to process all agents simultaneously. However, this
            # implementation is simpler and easier to understand, and the
            # greatest runtime usage is typically in the fitness function.

            # Pick random and distinct indices for agents in the population.
            a, b, c, k = tools.rand_choice(num_agents, size=4, replace=False)

            # Indices a, b, c, k are all different from each other.
            # Now ensure that indices a, b, c are also different from i.
            # Simply check if an index equals i and then replace it with k,
            # which is different from i, a, b, c.
            if i == a:
                a = k
            elif i == b:
                b = k
            elif i == c:
                c = k

            assert i != a != b != c

            # Original position in the search-space for the agent to be updated.
            original = population[i, :]

            # Calculate crossover of randomly selected agents from the population.
            crossover = population[a, :] + differential_weight * (population[b, :] - population[c, :])

            # Create a new agent (i.e. position) in the search-space from the crossover.
            # The crossover probability decides whether to use the crossover or
            # the agent's original position in the search-space.
            new_agent = np.where(tools.rand_uniform(dim) < crossover_probability, crossover, original)

            # Ensure at least one element of the agent's new position is from the crossover.
            rand_index = tools.rand_int(lower=0, upper=dim)
            new_agent[rand_index] = crossover[rand_index]

            # Bound the agent's new position to the search-space boundaries.
            new_agent = tools.bound(x=new_agent,
                                    lower=self.problem.lower_bound,
                                    upper=self.problem.upper_bound)

            # Assign the agent's new position to the temporary population so the
            # fitness can be calculated in another function.
            self.new_population[i, :] = new_agent

    def _fitness(self, i):
        """
        Calculate the fitness for the i'th agent.
        """

        # Note that self.new_population is used because they are the potentially
        # new positions for the agents.
        return self.problem.fitness(self.new_population[i, :], limit=self.fitness[i])

    def _update_fitness(self):
        """
        Calculate the fitness for each agent and update the agent's and population's
        best-known fitness and position if an improvement is found.
        """

        if not self.parallel:
            # Calculate the fitness for each agent. Not parallel.
            new_fitness = [self._fitness(i) for i in range(self.num_agents)]
        else:
            import multiprocessing as mp

            # Create a pool of workers sized according to the CPU cores available.
            pool = mp.Pool()

            # Calculate the fitness for each agent in parallel.
            new_fitness = pool.map(self._fitness, range(self.num_agents))

            # Close the pool of workers and wait for them all to finish.
            pool.close()
            pool.join()

        # For each agent in the population.
        for i in range(self.num_agents):
            # If the new fitness is an improvement over the agent's best-known fitness.
            if new_fitness[i] < self.fitness[i]:
                # Update the agent's best-known fitness and position.
                self.fitness[i] = new_fitness[i]
                self.population[i, :] = self.new_population[i, :]

                # Update the entire population's best-known fitness and position
                # if an improvement.
                # The parent-class is used for this.
                self._update_best(fitness=self.fitness[i],
                                  x=self.population[i, :])


##################################################



########################################################################
# SwarmOps - Heuristic optimization for Python.
# Copyright (C) 2003-2016 Magnus Erik Hvass Pedersen.
# See the file README.md for instructions.
# See the file LICENSE.txt for license details.
# SwarmOps on the internet: http://www.Hvass-Labs.org/
########################################################################

########################################################################
# Local Unimodal Sampling (LUS).
#
# Performs localized sampling of the search-space with a sampling range
# that initially covers the entire search-space and is decreased
# exponentially as optimization progresses. LUS works especially well
# for optimization problems where only short runs can be performed
# and the search-space is fairly smooth.
#
# References:
#
# [1] M.E.H. Pedersen. Tuning & Simplifying Heuristical Optimization (PhD thesis).
#     University of Southampton, School of Engineering Sciences. 2010
#     http://www.hvass-labs.org/people/magnus/thesis/pedersen08thesis.pdf
#
########################################################################

from swarmops.Optimize import SingleRun
from swarmops import tools


########################################################################

class LUS(SingleRun):
    """
        Perform a single optimization run using Local Unimodal Sampling (LUS).

        In practice, you would typically perform multiple optimization runs using
        the MultiRun-class. The reason is that LUS is a heuristic optimizer so
        there is no guarantee that an acceptable solution is found in any single
        run. It is more likely that an acceptable solution is found if you perform
        multiple optimization runs.
    """

    # Name of this optimizer.
    name = "LUS"
    name_full = "Local Unimodal Sampling"

    # Number of control parameters for LUS. Used by MetaFitness-class.
    num_parameters = 1

    # Lower boundaries for the control parameters of LUS. Used by MetaFitness-class.
    parameters_lower_bound = [0.1]

    # Upper boundaries for the control parameters of LUS. Used by MetaFitness-class.
    parameters_upper_bound = [100.0]

    @staticmethod
    def parameters_dict(parameters):
        """
        Create and return a dict from a list of LUS parameters.
        This is useful for printing the named parameters.

        :param parameters: List with LUS parameters assumed to be in the correct order.
        :return: Dict with LUS parameters.
        """

        return {'gamma': parameters[0]}

    @staticmethod
    def parameters_list(gamma):
        """
        Create a list with LUS parameters in the correct order.

        :param gamma: Gamma-parameter (see paper reference for explanation).
        :return: List with LUS parameters.
        """

        return [gamma]

    # Default parameters for LUS which will be used if no other parameters are specified.
    parameters_default = [3.0]

    def __init__(self, problem, parameters=parameters_default, parallel=False, *args, **kwargs):
        """
        Create object instance and perform a single optimization run using LUS.

        :param problem:
            The problem to be optimized. Instance of Problem-class.

        :param parameters:
            Control parameters for LUS.

        :param parallel: False. LUS cannot run in parallel except through MultiRun.

        :return:
            Object instance. Get the optimization results from the object's variables.
            -   best is the best-found solution.
            -   best_fitness is the associated fitness of the best-found solution.
            -   fitness_trace is an instance of the FitnessTrace-class.
        """

        # Copy arguments to instance variables.
        self.problem = problem

        # Unpack control parameters.
        gamma = parameters[0]

        # Derived control parameter.
        self.decrease_factor = 0.5 ** (1.0 / (gamma * problem.dim))

        # Initialize parent-class which also starts the optimization run.
        SingleRun.__init__(self, *args, **kwargs)

    def _optimize(self):
        """
        Perform a single optimization run.
        This function is called by the parent-class.
        """

        # Convenience variable for fitness function.
        f = self.problem.fitness

        # Convenience variables for search-space boundaries.
        lower_init = self.problem.lower_init
        upper_init = self.problem.upper_init
        lower_bound = self.problem.lower_bound
        upper_bound = self.problem.upper_bound

        # Initialize the range-vector to full search-space.
        d = upper_bound - lower_bound

        # Search-space dimensionality.
        dim = self.problem.dim

        # Initialize x with random position in search-space.
        x = tools.rand_array(lower=lower_init, upper=upper_init)

        # Compute fitness of initial position.
        fitness = f(x)

        # Update the best-known fitness and position.
        # The parent-class is used for this.
        self._update_best(fitness=fitness, x=x)

        # Perform optimization iterations until the maximum number
        # of fitness evaluations has been performed.
        # Count starts at one because we have already calculated fitness once above.
        evaluations = 1
        while evaluations < self.max_evaluations:
            # Sample new position y from the bounded surroundings
            # of the current position x.
            y = tools.sample_bounded(x=x, d=d, lower=lower_bound, upper=upper_bound)

            # Compute new fitness.
            new_fitness = f(y, limit=fitness)

            # If improvement to fitness.
            if new_fitness < fitness:
                # Update fitness and position.
                fitness = new_fitness
                x = y

                # Update the best-known fitness and position.
                # The parent-class is used for this.
                self._update_best(fitness=fitness, x=x)
            else:
                # Otherwise decrease the search-range.
                d *= self.decrease_factor

            # Call parent-class to print status etc. during optimization.
            self._iteration(evaluations)

            # Increment counter.
            evaluations += 1

########################################################################



########################################################################
# SwarmOps - Heuristic optimization for Python.
# Copyright (C) 2003-2016 Magnus Erik Hvass Pedersen.
# See the file README.md for instructions.
# See the file LICENSE.txt for license details.
# SwarmOps on the internet: http://www.Hvass-Labs.org/
########################################################################

########################################################################
# Particle Swarm Optimization (PSO).
#
# PSO is a heuristic optimizer that does not use the gradient of the problem
# being optimized. A so-called global-best variant of the PSO is implemented here.
# A simple PSO variant is also implemented here.
#
# Search-space boundaries are necessary for this PSO variant to work properly.
# So if your optimization problem does not have natural boundaries, you should
# simply choose some boundaries that are reasonable.
#
# PSO starts by creating a number of random trials called particles. In each
# iteration, these particles are moved around in the search-space using a
# formula that involves the particle's best-known position as well as the
# entire swarm's best-known position. This has been found to work well for
# optimizing many difficult problems, although a satisfactory solution is
# not guaranteed to be found.
#
# The PSO was originally proposed around year 1995, see [1] and [2]. In the
# following 20 years, thousands of PSO variants have been proposed.
# One of the early and basic variants of the PSO is implemented here.
# Newer PSO variants often claim to adapt the control parameters during
# optimization, thus making the PSO adapt better to new problems. But it
# was found in [3] that the basic PSO could perform just as well if using
# proper control parameters. Control parameters tuned for different
# optimization scenarios are given in [4] and included in this file below.
#
# References:
#
# [1] J. Kennedy, R.C. Eberhart. Particle Swarm Optimization. Proceedings of
#     IEEE International Conference on Neural Networks. pp. 1942-1948. 1995.
#
# [2] Y. Shi, R.C. Eberhart. A modified particle swarm optimizer. Proceedings
#     of IEEE International Conference on Evolutionary Computation. pp. 69-73. 1998.
#
# [3] M.E.H. Pedersen. Tuning & Simplifying Heuristical Optimization (PhD thesis).
#     University of Southampton, School of Engineering Sciences. 2010
#     http://www.hvass-labs.org/people/magnus/thesis/pedersen08thesis.pdf
#
# [4] M.E.H. Pedersen. Good parameters for particle swarm optimization.
#     Technical Report HL-1001, Hvass Laboratories. 2010.
#     http://www.hvass-labs.org/people/magnus/publications/pedersen10good-pso.pdf
#
########################################################################

import numpy as np
from swarmops.Optimize import SingleRun
from swarmops import tools


##################################################

class Base(SingleRun):
    def __init__(self, problem, parallel=False, *args, **kwargs):
        """
        Create object instance and perform a single optimization run using PSO.

        :param problem: The problem to be optimized. Instance of Problem-class.

        :param parallel:
            Evaluate the fitness for the particles in parallel.
            See the README.md file for a discussion on this.

        :return:
            Object instance. Get the optimization results from the object's variables.
            -   best is the best-found solution.
            -   best_fitness is the associated fitness of the best-found solution.
            -   fitness_trace is an instance of the FitnessTrace-class.
        """

        # Copy arguments to instance variables.
        self.problem = problem
        self.parallel = parallel

        # Initialize all particles with random positions in the search-space.
        # The first index is for the particle number.
        # The second index is for the search-space dimension.
        # Note that self.num_particles must be set prior to this by the sub-class.
        self.particle = tools.rand_population(lower=problem.lower_init,
                                              upper=problem.upper_init,
                                              num_agents=self.num_particles,
                                              dim=problem.dim)

        # Initialize best-known positions for the particles to their starting positions.
        # A copy is made because the particle positions will change during optimization
        # regardless of improvement to the particle's fitness.
        self.particle_best = np.copy(self.particle)

        # Initialize fitness of best-known particle positions to infinity.
        self.particle_best_fitness = np.repeat(np.inf, self.num_particles)

        # Boundaries for the velocity. These are set to the range of the search-space.
        bound_range = np.abs(problem.upper_bound - problem.lower_bound)
        self.velocity_lower_bound = -bound_range
        self.velocity_upper_bound = bound_range

        # Initialize all velocities with random values in the allowed range.
        self.velocity = tools.rand_population(lower=self.velocity_lower_bound,
                                              upper=self.velocity_upper_bound,
                                              num_agents=self.num_particles,
                                              dim=problem.dim)

        # Initialize parent-class which also starts the optimization run.
        SingleRun.__init__(self, *args, **kwargs)

    def _optimize(self):
        """
        Perform a single optimization run.
        This function is called by the parent-class.
        """

        # Calculate fitness for the initial particle positions.
        self._update_fitness()

        # Optimization iterations.
        # The counting starts with num_particles because the fitness has
        # already been calculated once for each particle during initialization.
        for i in range(self.num_particles, self.max_evaluations, self.num_particles):
            # Update the particle velocities and positions.
            self._update_particles()

            # Update the fitness for each particle.
            self._update_fitness()

            # Call parent-class to print status etc. during optimization.
            self._iteration(i)

    def _fitness(self, i):
        """
        Calculate the fitness for the i'th particle.
        """

        return self.problem.fitness(self.particle[i, :], limit=self.particle_best_fitness[i])

    def _update_fitness(self):
        """
        Calculate and update the fitness for each particle. Also updates the particle's
        and swarm's best-known fitness and position if an improvement is found.
        """

        if not self.parallel:
            # Calculate the fitness for each particle. Not parallel.
            new_fitness = [self._fitness(i) for i in range(self.num_particles)]
        else:
            import multiprocessing as mp

            # Create a pool of workers sized according to the CPU cores available.
            pool = mp.Pool()

            # Calculate the fitness for each particle in parallel.
            new_fitness = pool.map(self._fitness, range(self.num_particles))

            # Close the pool of workers and wait for them all to finish.
            pool.close()
            pool.join()

        # For each particle.
        for i in range(self.num_particles):
            # If the fitness is an improvement over the particle's best-known fitness.
            if new_fitness[i] < self.particle_best_fitness[i]:
                # Update the particle's best-known fitness and position.
                self.particle_best_fitness[i] = new_fitness[i]
                self.particle_best[i, :] = self.particle[i, :]

                # Update the entire swarm's best-known fitness and position if an improvement.
                # The parent-class is used for this.
                self._update_best(fitness=new_fitness[i],
                                  x=self.particle_best[i, :])


##################################################

class PSO(Base):
    """
        Perform a single optimization run using Particle Swarm Optimization (PSO).

        This is a so-called global-best variant, although it may have slightly
        different features than other global-best variants in the research literature.

        In practice, you would typically perform multiple optimization runs using
        the MultiRun-class. The reason is that PSO is a heuristic optimizer so
        there is no guarantee that an acceptable solution is found in any single
        run. It is more likely that an acceptable solution is found if you perform
        multiple optimization runs.

        Control parameters have been tuned for different optimization scenarios.
        First try and use the default parameters. If that does not give
        satisfactory results, then you may try some of the following.
        Select the parameters that most closely match your problem.
        For example, if you want to optimize a problem where the search-space
        has 15 dimensions and you can perform 30000 evaluations, then you could
        first try using parameters_20dim_40000eval. If that does not give
        satisfactory results then you could try using parameters_10dim_20000eval.
        If that does not work then you will either need to meta-optimize the
        parameters for the problem at hand, or you should try using another optimizer.
    """

    # Name of this optimizer.
    name = "PSO"
    name_full = "Particle Swarm Optimization (Global-Best Variant)"

    # Number of control parameters for PSO. Used by MetaFitness-class.
    num_parameters = 4

    # Lower boundaries for the control parameters of PSO. Used by MetaFitness-class.
    parameters_lower_bound = [1.0, -2.0, -4.0, -4.0]

    # Upper boundaries for the control parameters of PSO. Used by MetaFitness-class.
    parameters_upper_bound = [300.0, 2.0, 4.0, 6.0]

    @staticmethod
    def parameters_list(num_particles, omega, phi_p, phi_g):
        """
        Create a list with PSO parameters in the correct order.

        :param num_particles: Number of particles for the PSO swarm.
        :param omega: The omega parameter (aka. inertia weight) for the PSO.
        :param phi_p: The phi_p parameter (aka. particle weight) for the PSO.
        :param phi_g: The phi_g parameter (aka. social weight) for the PSO.
        :return: List with PSO parameters.
        """

        return [num_particles, omega, phi_p, phi_g]

    @staticmethod
    def parameters_dict(parameters):
        """
        Create and return a dict from a list of PSO parameters.
        This is useful for printing the named parameters.

        :param parameters: List with PSO parameters assumed to be in the correct order.
        :return: Dict with PSO parameters.
        """

        return {'num_particles': parameters[0],
                'omega': parameters[1],
                'phi_p': parameters[2],
                'phi_g': parameters[3]}

    # Default parameters for the PSO which will be used if no other parameters are specified.
    # These are a compromise of the tuned parameters below. Try this first and see if it works.
    parameters_default = [50.0, -0.4, -0.3, 3.9]

    # Parameters tuned by hand. These are common in the older research literature on PSO
    # but perform much worse than meta-optimized parameters, especially for this PSO variant.
    parameters_hand_tuned = [50.0, 0.729, 1.49445, 1.49445]

    # Parameters tuned for benchmark problems in 2 dimensions using 400 fitness evaluations.
    parameters_2dim_400eval_a = [25.0, 0.3925, 2.5586, 1.3358]
    parameters_2dim_400eval_b = [29.0, -0.4349, -0.6504, 2.2073]

    # Parameters tuned for benchmark problems in 2 dimensions using 4000 fitness evaluations.
    parameters_2dim_4000eval_a = [156.0, 0.4091, 2.1304, 1.0575]
    parameters_2dim_4000eval_b = [237.0, -0.2887, 0.4862, 2.5067]

    # Parameters tuned for benchmark problems in 5 dimensions using 1000 fitness evaluations.
    parameters_5dim_1000eval_a = [63.0, -0.3593, -0.7238, 2.0289]
    parameters_5dim_1000eval_b = [47.0, -0.1832, 0.5287, 3.1913]

    # Parameters tuned for benchmark problems in 5 dimensions using 10000 fitness evaluations.
    parameters_5dim_10000eval_a = [223.0, -0.3699, -0.1207, 3.3657]
    parameters_5dim_10000eval_b = [203.0, 0.5069, 2.5524, 1.0056]

    # Parameters tuned for benchmark problems in 10 dimensions using 2000 fitness evaluations.
    parameters_10dim_2000eval_a = [63.0, 0.6571, 1.6319, 0.6239]
    parameters_10dim_2000eval_b = [204.0, -0.2134, -0.3344, 2.3259]

    # Parameters tuned for benchmark problems in 10 dimensions using 20000 fitness evaluations.
    parameters_10dim_20000eval = [53.0, -0.3488, -0.2746, 4.8976]

    # Parameters tuned for benchmark problems in 20 dimensions using 40000 fitness evaluations.
    parameters_20dim_40000eval = [69.0, -0.4438, -0.2699, 3.395]

    # Parameters tuned for benchmark problems in 20 dimensions using 400000 fitness evaluations.
    parameters_20dim_400000eval_a = [149.0, -0.3236, -0.1136, 3.9789]
    parameters_20dim_400000eval_b = [60.0, -0.4736, -0.97, 3.7904]
    parameters_20dim_400000eval_c = [256.0, -0.3499, -0.0513, 4.9087]

    # Parameters tuned for benchmark problems in 30 dimensions using 60000 fitness evaluations.
    parameters_30dim_60000eval = [134.0, -0.1618, 1.8903, 2.1225]

    # Parameters tuned for benchmark problems in 30 dimensions using 600000 fitness evaluations.
    parameters_30dim_600000eval = [95.0, -0.6031, -0.6485, 2.6475]

    # Parameters tuned for benchmark problems in 50 dimensions using 100000 fitness evaluations.
    parameters_50dim_100000eval = [106.0, -0.2256, -0.1564, 3.8876]

    # Parameters tuned for benchmark problems in 100 dimensions using 200000 fitness evaluations.
    parameters_100dim_200000eval = [161.0, -0.2089, -0.0787, 3.7637]

    def __init__(self, parameters=parameters_default, *args, **kwargs):
        """
        Create object instance and perform a single optimization run using PSO.

        :param parameters:
            Control parameters for the PSO.
            These may have a significant impact on the optimization performance.
            First try and use the default parameters and if they don't give satisfactory
            results, then experiment with other parameters.

        :return:
            Object instance. Get the optimization results from the object's variables.
            -   best is the best-found solution.
            -   best_fitness is the associated fitness of the best-found solution.
            -   fitness_trace is an instance of the FitnessTrace-class.
        """

        # Unpack control parameters.
        self.num_particles, self.omega, self.phi_p, self.phi_g = parameters

        # The number of particles must be an integer.
        self.num_particles = int(self.num_particles)

        # Initialize parent-class which also starts the optimization run.
        Base.__init__(self, *args, **kwargs)

    def _update_particles(self):
        """
        Update the velocities and positions for all particles.
        This does not update the fitness for each particle.
        """

        # Random values between zero and one. One random value per particle.
        rand_p = tools.rand_uniform(size=self.num_particles)
        rand_g = tools.rand_uniform(size=self.num_particles)

        # Update velocity for all particles using numpy operations.
        # For an explanation of this formula, see the research papers referenced above.
        # Note that self.best is the swarm's best-known position aka. global-best.
        self.velocity = (self.omega * self.velocity.T \
                         + self.phi_p * rand_p * (self.particle_best - self.particle).T \
                         + self.phi_g * rand_g * (self.best - self.particle).T).T

        # Fix de-normalized floating point values which can make the execution very slow.
        self.velocity = tools.denormalize_trunc(self.velocity)

        # Bound velocity.
        self.velocity = tools.bound(self.velocity, self.velocity_lower_bound, self.velocity_upper_bound)

        # Update particle positions in the search-space by adding the velocity.
        self.particle = self.particle + self.velocity

        # Bound particle position to search-space.
        self.particle = tools.bound(self.particle, self.problem.lower_bound, self.problem.upper_bound)


##################################################

class MOL(Base):
    """
        Perform a single optimization run using Many Optimizing Liaisons (MOL).

        In practice, you would typically perform multiple optimization runs using
        the MultiRun-class. The reason is that MOL is a heuristic optimizer so
        there is no guarantee that an acceptable solution is found in any single
        run. It is more likely that an acceptable solution is found if you perform
        multiple optimization runs.

        Control parameters have been tuned for different optimization scenarios.
        First try and use the default parameters. If that does not give
        satisfactory results, then you may try some of the following.
        Select the parameters that most closely match your problem.
        For example, if you want to optimize a problem where the search-space
        has 15 dimensions and you can perform 30000 evaluations, then you could
        first try using parameters_20dim_40000eval. If that does not give
        satisfactory results then you could try using parameters_10dim_20000eval.
        If that does not work then you will either need to meta-optimize the
        parameters for the problem at hand, or you should try using another optimizer.
    """

    # Name of this optimizer.
    name = "MOL"
    name_full = "Many Optimizing Liaisons (Simple Variant of PSO)"

    # Number of control parameters for MOL. Used by MetaFitness-class.
    num_parameters = 3

    # Lower boundaries for the control parameters of MOL. Used by MetaFitness-class.
    parameters_lower_bound = [1.0, -2.0, -4.0]

    # Upper boundaries for the control parameters of MOL. Used by MetaFitness-class.
    parameters_upper_bound = [300.0, 2.0, 6.0]

    @staticmethod
    def parameters_dict(parameters):
        """
        Create and return a dict from a list of MOL parameters.
        This is useful for printing the named parameters.

        :param parameters: List with MOL parameters assumed to be in the correct order.
        :return: Dict with MOL parameters.
        """

        return {'num_particles': parameters[0],
                'omega': parameters[1],
                'phi_g': parameters[2]}

    @staticmethod
    def parameters_list(num_particles, omega, phi_g):
        """
        Create a list with MOL parameters in the correct order.

        :param num_particles: Number of particles for the MOL swarm.
        :param omega: The omega parameter (aka. inertia weight) for the MOL.
        :param phi_g: The phi_g parameter (aka. social weight) for the MOL.
        :return: List with MOL parameters.
        """

        return [num_particles, omega, phi_g]

    # Default parameters for MOL which will be used if no other parameters are specified.
    # These are a compromise of the tuned parameters below. Try this first and see if it works.
    parameters_default = [100.0, -0.35, 3.0]

    # Parameters tuned for benchmark problems in 2 dimensions using 400 fitness evaluations.
    parameters_2dim_400eval_a = [23.0, -0.3328, 2.8446]
    parameters_2dim_400eval_b = [50.0,  0.2840, 1.9466]

    # Parameters tuned for benchmark problems in 2 dimensions using 4000 fitness evaluations.
    parameters_2dim_4000eval_a = [183.0, -0.2797, 3.0539]
    parameters_2dim_4000eval_b = [139.0,  0.6372, 1.0949]

    # Parameters tuned for benchmark problems in 5 dimensions using 10000 fitness evaluations.
    parameters_5dim_1000eval = [50.0, -0.3085, 2.0273]

    # Parameters tuned for benchmark problems in 5 dimensions using 10000 fitness evaluations.
    parameters_5dim_10000eval = [96.0, -0.3675, 4.1710]

    # Parameters tuned for benchmark problems in 10 dimensions using 2000 fitness evaluations.
    parameters_10dim_2000eval = [60.0, -0.2700, 2.9708]

    # Parameters tuned for benchmark problems in 10 dimensions using 20000 fitness evaluations.
    parameters_10dim_20000eval = [116.0, -0.3518, 3.8304]

    # Parameters tuned for benchmark problems in 20 dimensions using 40000 fitness evaluations.
    parameters_20dim_40000eval = [228.0, -0.3747, 4.2373]

    # Parameters tuned for benchmark problems in 20 dimensions using 400000 fitness evaluations.
    parameters_20dim_400000eval = [125.0, -0.2575, 4.6713]

    # Parameters tuned for benchmark problems in 30 dimensions using 600000 fitness evaluations.
    parameters_30dim_60000eval = [198.0, -0.2723, 3.8283]

    # Parameters tuned for benchmark problems in 50 dimensions using 100000 fitness evaluations.
    parameters_50dim_100000eval = [290.0, -0.3067, 3.6223]

    # Parameters tuned for benchmark problems in 100 dimensions using 200000 fitness evaluations.
    parameters_100dim_200000eval = [219.0, -0.1685, 3.9162]

    def __init__(self, parameters=parameters_default, *args, **kwargs):
        """
        Create object instance and perform a single optimization run using MOL.

        :param problem: The problem to be optimized. Instance of Problem-class.

        :param parameters:
            Control parameters for the MOL.
            These may have a significant impact on the optimization performance.
            First try and use the default parameters and if they don't give satisfactory
            results, then experiment with other the parameters.

        :return:
            Object instance. Get the optimization results from the object's variables.
            -   best is the best-found solution.
            -   best_fitness is the associated fitness of the best-found solution.
            -   fitness_trace is an instance of the FitnessTrace-class.
        """

        # Unpack control parameters.
        self.num_particles, self.omega, self.phi_g = parameters

        # The number of particles must be an integer.
        self.num_particles = int(self.num_particles)

        # Initialize parent-class which also starts the optimization run.
        Base.__init__(self, *args, **kwargs)

    def _update_particles(self):
        """
        Update the velocities and positions for all particles.
        This does not update the fitness for each particle.
        """

        # Random values between zero and one. One random value per particle.
        rand_g = tools.rand_uniform(size=self.num_particles)

        # Update velocity for all particles using numpy operations.
        # For an explanation of this formula, see the research papers referenced above.
        # Note that self.best is the swarm's best-known position aka. global-best.
        self.velocity = (self.omega * self.velocity.T \
                         + self.phi_g * rand_g * (self.best - self.particle).T).T

        # Fix de-normalized floating point values which can make the execution very slow.
        self.velocity = tools.denormalize_trunc(self.velocity)

        # Bound velocity.
        self.velocity = tools.bound(self.velocity, self.velocity_lower_bound, self.velocity_upper_bound)

        # Update particle positions in the search-space by adding the velocity.
        self.particle = self.particle + self.velocity

        # Bound particle position to search-space.
        self.particle = tools.bound(self.particle, self.problem.lower_bound, self.problem.upper_bound)


##################################################



########################################################################
# SwarmOps - Heuristic optimization for Python.
# Copyright (C) 2003-2016 Magnus Erik Hvass Pedersen.
# See the file README.md for instructions.
# See the file LICENSE.txt for license details.
# SwarmOps on the internet: http://www.Hvass-Labs.org/
########################################################################

########################################################################
# Problem-class used for implementing optimization problems. This
# essentially just wraps the fitness-function, boundaries, etc.
#
# This file also implements several common benchmark problems whose
# mathematical definitions are given in this paper:
#
# [1] M.E.H. Pedersen. Good parameters for particle swarm optimization.
#     Technical Report HL-1001, Hvass Laboratories. 2010.
#     http://www.hvass-labs.org/people/magnus/publications/pedersen10good-pso.pdf
#
########################################################################

import numpy as np
from swarmops import tools


########################################################################

class Problem:
    """
    Implements optimization problems by wrapping the fitness-function,
    boundaries, etc.
    """

    def __init__(self, name, dim, fitness_min,
                 lower_bound, upper_bound,
                 lower_init=None, upper_init=None,
                 func=None):
        """
        Create the object instance for an optimization problem.

        You can implement a fitness function in either of two ways:

        1) You can sub-class this Problem-class and override the self.fitness() function.
           This is demonstrated in the benchmark problems below.

        2) You can pass the fitness function as the func-argument to __init__.
           The arguments of the fitness function must match those of self.fitness().
           Note that if you want to do parallel execution then the fitness function
           must not be declared inside "if __name__ == '__main__':" due to some
           peculiarities in how multiprocessing uses pickle.
           This is demonstrated in demo-optimize.py

        :param name: Name of the optimization problem.
        :param dim: Dimensionality of the search-space.
        :param fitness_min: Minimum possible fitness value, or a reasonable lower boundary.
        :param lower_bound: Lower boundary for the search-space.
        :param upper_bound: Upper boundary for the search-space.
        :param lower_init: Lower initialization boundary (if None then user lower_bound).
        :param upper_init: Upper initialization boundary (if None then user upper_bound).
        :param func: Wrap a fitness-function whose arguments must match self.fitness().
        """

        # Copy arguments to instance variables.
        self.name = name
        self.name_full = "{0} ({1} dim)".format(name, dim)
        self.dim = dim
        self.fitness_min = fitness_min

        # If a fitness function is supplied as argument,
        # then use it instead of the class method.
        if func is not None:
            self.fitness = func

        # Boundaries for the search-space. Ensure they are numpy float arrays.
        self.lower_bound = np.array(lower_bound, dtype='float64')
        self.upper_bound = np.array(upper_bound, dtype='float64')

        # Lower initialization boundary.
        if lower_init is not None:
            # Use the initialization boundary provided.
            self.lower_init = np.array(lower_init, dtype='float64')
        else:
            # Or use the search-space boundary.
            self.lower_init = self.lower_bound

        # Upper initialization boundary.
        if upper_init is not None:
            # Use the initialization boundary provided.
            self.upper_init = np.array(upper_init, dtype='float64')
        else:
            # Or use the search-space boundary.
            self.upper_init = self.upper_bound

        # Ensure boundaries have the correct dimensions.
        assert dim == len(self.lower_bound) == len(self.upper_bound)
        assert dim == len(self.lower_init) == len(self.upper_init)

    def fitness(self, x, limit=np.Infinity):
        """
        This is the fitness function that must be implemented for an
        optimization problem.

        It is also sometimes called the cost- or error-function.

        :param x:
            Calculate the fitness value for these parameters in the search-space.

        :param limit:
            Calculation of the fitness can be aborted if the value is greater than this limit.
            This is used for so-called Pre-Emptive Fitness Evaluation in the MetaFitness-class.
            You can ignore this value.

        :return:
            Fitness-value of x.
        """

        # Raise an exception if the child-class has not implemented this function.
        raise NotImplementedError


########################################################################

class Benchmark(Problem):
    """
    Same as the Problem-class except the boundaries are assumed to be scalar
    values used for creating numpy arrays. This is convenient for optimization
    problems whose boundaries are identical for all dimensions, such as
    the benchmark-problems below.
    """

    def __init__(self, dim, lower_bound, upper_bound, lower_init, upper_init,
                 *args, **kwargs):
        """
        Create the object instance for an optimization problem.

        :param dim: Search-space dimensionality.
        :param lower_bound: Scalar value for the lower search-space boundary.
        :param upper_bound: Scalar value for the upper search-space boundary.
        :param lower_init: Scalar value for the lower initialization boundary.
        :param upper_init: Scalar value for the upper initialization boundary.
        :param kwargs:
        :return:
        """

        Problem.__init__(self, dim=dim,
                         lower_bound=np.repeat(lower_bound, dim),
                         upper_bound=np.repeat(upper_bound, dim),
                         lower_init=np.repeat(lower_init, dim),
                         upper_init=np.repeat(upper_init, dim),
                         *args, **kwargs)


########################################################################
# Benchmark problems.

class Sphere(Benchmark):
    """
    The Sphere benchmark problem.
    """

    def __init__(self, dim):
        Benchmark.__init__(self, name='Sphere', dim=dim, fitness_min=0.0,
                           lower_bound=-100.0, upper_bound=100.0,
                           lower_init=50.0, upper_init=100.0)

    def fitness(self, x, limit=np.Infinity):
        return np.sum(x ** 2)

class Rosenbrock(Benchmark):
    """
    The Rosenbrock benchmark problem.
    """

    def __init__(self, dim):
        Benchmark.__init__(self, name='Rosenbrock', dim=dim, fitness_min=0.0,
                           lower_bound=-100.0, upper_bound=100.0,
                           lower_init=15.0, upper_init=30.0)

    def fitness(self, x, limit=np.Infinity):
        return np.sum(100.0 * (x[1:] - x[:-1] ** 2) ** 2 + (1.0 - x[:-1]) ** 2)


class Griewank(Benchmark):
    """
    The Griewank benchmark problem.
    """

    def __init__(self, dim):
        Benchmark.__init__(self, name='Griewank', dim=dim, fitness_min=0.0,
                           lower_bound=-600.0, upper_bound=600.0,
                           lower_init=300.0, upper_init=600.0)

        # Helper array for faster calculation of fitness.
        # This is the square root of the array index.
        self.sqrt_idx = np.sqrt(np.linspace(start=1.0, stop=dim, num=dim))

    def fitness(self, x, limit=np.Infinity):
        return 1.0 + np.sum(x ** 2) / 4000.0 - np.prod(np.cos(x / self.sqrt_idx))


class Rastrigin(Benchmark):
    """
    The Rastrigin benchmark problem.
    """

    def __init__(self, dim):
        Benchmark.__init__(self, name='Rastrigin', dim=dim, fitness_min=0.0,
                           lower_bound=-5.12, upper_bound=5.12,
                           lower_init=2.56, upper_init=5.12)

    def fitness(self, x, limit=np.Infinity):
        return np.sum(x ** 2 + 10.0 - 10.0 * np.cos(2 * np.pi * x))


class Schwefel1_2(Benchmark):
    """
    The Schwefel 1.2 benchmark problem.
    """

    def __init__(self, dim):
        Benchmark.__init__(self, name='Schwefel 1-2', dim=dim, fitness_min=0.0,
                           lower_bound=-30.0, upper_bound=30.0,
                           lower_init=15.0, upper_init=30.0)

    def fitness(self, x, limit=np.Infinity):
        return np.sum([np.sum(x[:i])**2 for i in range(self.dim)])


class Schwefel2_21(Benchmark):
    """
    The Schwefel 2.21 benchmark problem.
    """

    def __init__(self, dim):
        Benchmark.__init__(self, name='Schwefel 2-21', dim=dim, fitness_min=0.0,
                           lower_bound=-100.0, upper_bound=100.0,
                           lower_init=50.0, upper_init=100.0)

    def fitness(self, x, limit=np.Infinity):
        return np.max(np.abs(x))


class Schwefel2_22(Benchmark):
    """
    The Schwefel 2.22 benchmark problem.
    """

    def __init__(self, dim):
        Benchmark.__init__(self, name='Schwefel 2-22', dim=dim, fitness_min=0.0,
                           lower_bound=-10.0, upper_bound=10.0,
                           lower_init=5.0, upper_init=10.0)

    def fitness(self, x, limit=np.Infinity):
        absx = np.abs(x)
        fitness = np.sum(absx) + np.prod(absx)

        # For large dim, np.prod(absx) often overflows to infinity.
        # This means the optimizers cannot work properly.
        # So we limit the fitness to 1e20.
        if fitness == np.Infinity or fitness > 1e20:
            fitness = 1e20

        return fitness


class Step(Benchmark):
    """
    The Step benchmark problem.
    """

    def __init__(self, dim):
        Benchmark.__init__(self, name='Step', dim=dim, fitness_min=0.0,
                           lower_bound=-100.0, upper_bound=100.0,
                           lower_init=50.0, upper_init=100.0)

    def fitness(self, x, limit=np.Infinity):
        return np.sum(np.floor(x + 0.5) ** 2)


class QuarticNoise(Benchmark):
    """
    The Quartic-Noise benchmark problem.
    """

    def __init__(self, dim):
        Benchmark.__init__(self, name='QuarticNoise', dim=dim, fitness_min=0.0,
                           lower_bound=-1.28, upper_bound=1.28,
                           lower_init=0.64, upper_init=1.28)

        # Helper array with the index.
        self.idx_array = np.linspace(start=1.0, stop=dim, num=dim)

    def fitness(self, x, limit=np.Infinity):
        # If using PSO, MOL or DE in parallel mode and not run through MultiRun,
        # then a new PRNG must be created for each thread because the PRNG is not
        # thread-safe. This is a bit slow so it should only be done when necessary.
        if False:
            tools.new_prng()

        # Array with random values between zero and one.
        r = tools.rand_uniform(size=self.dim)

        return np.sum(self.idx_array * (x ** 4) + r)


class Ackley(Benchmark):
    """
    The Ackley benchmark problem.
    """

    def __init__(self, dim):
        Benchmark.__init__(self, name='Ackley', dim=dim, fitness_min=0.0,
                           lower_bound=-30.0, upper_bound=30.0,
                           lower_init=15.0, upper_init=30.0)

    def fitness(self, x, limit=np.Infinity):
        value = np.e + 20.0 - 20.0 * np.exp(-0.2 * np.sqrt(np.sum(x ** 2) / self.dim)) \
                - np.exp(np.sum(np.cos(2 * np.pi * x)) / self.dim)

        # Due to rounding errors, the value is sometimes slightly negative.
        # This gives a warning in the MetaFitness-class.
        if value < 0.0:
            value = 0.0

        return value


def _penalty(x, a, k, m):
    """
    Helper-function for the Penalized-problems below.
    """

    # Penalty for each dimension of the search-space.
    p = np.where(x < -a, k * (-x - a)**m,
                 np.where(x > a, k * (x - a)**m, 0.0))

    # Sum the penalties.
    return np.sum(p)


class Penalized1(Benchmark):
    """
    The Penalized1 benchmark problem.
    """

    def __init__(self, dim):
        Benchmark.__init__(self, name='Penalized1', dim=dim, fitness_min=0.0,
                           lower_bound=-50.0, upper_bound=50.0,
                           lower_init=5.0, upper_init=50.0)

    def fitness(self, x, limit=np.Infinity):
        # Split the calculation into smaller formulas.
        y = 1.0 + (x + 1.0) / 4.0
        a = 10.0 * np.sin(np.pi * y[0]) ** 2
        b = np.sum(((y[:-1] - 1.0) ** 2) * (1.0 + 10.0 * (np.sin(np.pi * y[1:]) ** 2)))
        c = (y[-1] - 1.0) ** 2

        # Penalty term.
        penalty = _penalty(x, 10.0, 100.0, 4.0)

        return np.pi * (a + b + c) / self.dim + penalty


class Penalized2(Benchmark):
    """
    The Penalized2 benchmark problem.
    """

    def __init__(self, dim):
        Benchmark.__init__(self, name='Penalized2', dim=dim, fitness_min=0.0,
                           lower_bound=-50.0, upper_bound=50.0,
                           lower_init=5.0, upper_init=50.0)

    def fitness(self, x, limit=np.Infinity):
        # Split the calculation into smaller formulas.
        a = np.sin(3 * np.pi * x[0]) ** 2
        b = np.sum(((x[:-1] - 1.0) ** 2) * (1.0 + np.sin(3 * np.pi * x[1:]) ** 2))
        c = (x[-1] - 1.0) ** 2 * (1.0 + np.sin(2 * np.pi * x[-1])**2)

        # Penalty term.
        penalty = _penalty(x, 5.0, 100.0, 4.0)

        return 0.1 * (a + b + c) / self.dim + penalty


########################################################################

# List of all of the above classes for benchmark problems.
all_benchmark_problems = [Ackley, Griewank, Penalized1, Penalized2,
                          QuarticNoise, Rastrigin, Rosenbrock,
                          Schwefel1_2, Schwefel2_21, Schwefel2_22,
                          Sphere, Step]

########################################################################



########################################################################
# SwarmOps - Heuristic optimization for Python.
# Copyright (C) 2003-2016 Magnus Erik Hvass Pedersen.
# See the file README.md for instructions.
# See the file LICENSE.txt for license details.
# SwarmOps on the internet: http://www.Hvass-Labs.org/
########################################################################

########################################################################
# Classes used for tuning the control parameters of an optimizer.
#
# The basic idea is to have two layers of optimization, the optimizer
# whose control parameters we wish to tune (e.g. PSO or DE), and a
# meta-optimizer (typically LUS) for meta-optimizing those control
# parameters.
#
# The meta-fitness measures the performance of the optimizer on several
# optimization problems. It is basically just the sum of the best
# fitness achieved in several optimization runs on each problem.
#
# To save execution time, the calculation of the meta-fitness may be
# aborted pre-emptively if the fitness sum exceeds a limit.
# This is known as Pre-Emptive Fitness Evaluation.
#
# Meta-optimization is described in detail in:
#
# [1] M.E.H. Pedersen. Tuning & Simplifying Heuristical Optimization (PhD thesis).
#     University of Southampton, School of Engineering Sciences. 2010
#     http://www.hvass-labs.org/people/magnus/thesis/pedersen08thesis.pdf
#
########################################################################

import numpy as np
from swarmops.LogSolutions import LogSolutions
from swarmops.Optimize import MultiRun
from swarmops.Problem import Problem
from swarmops.Timer import Timer
from swarmops.LUS import LUS


########################################################################

class _ProblemRank:
    """
    Helper-class for ranking and sorting the list of problems according
    to how well an optimization method performed on each problem.

    This is used with Pre-Emptive Fitness Evaluation for more quickly
    aborting the computation of the meta-fitness measure when possible.
    """

    def __init__(self, problem, weight=1.0):
        """
        Create object instance.

        :param problem: Instance of the Problem-class.
        :param weight: Weight for the problem.
        :return: Object instance.
        """

        # Copy arguments to instance variables.
        self.problem = problem
        self.weight = weight

        # Initialize the fitness-sum.
        self.fitness_sum = 0.0

        # Initialize the best-found solution and fitness.
        self.best = None
        self.best_fitness = np.Infinity

    def update_best(self, best, best_fitness):
        """
        Update the best-found solution to the problem.

        :param best: Best-found solution from an optimization run.
        :param best_fitness: Fitness of the best-found solution.
        :return: Nothing.
        """

        # If the fitness is an improvement over the best-known.
        if best_fitness < self.best_fitness:
            # Update the best-known solution and fitness.
            self.best = best
            self.best_fitness = best_fitness


########################################################################

class _MetaFitness(Problem):
    """
    Used for measuring the performance of an optimization method on
    several problems. This is called the meta-fitness which can
    then be optimized by another overlaying optimizer which is called
    the meta-optimizer.
    """

    def __init__(self, optimizer, problems, num_runs, max_evaluations, weights=None):
        """
        Create object instance.

        :param optimizer: Optimizer-class, e.g. PSO, MOL or DE.
        :param problems: List of instances of the Problem-class.
        :param num_runs: Number of optimization runs to perform for each problem.
        :param max_evaluations: Number of fitness evaluations for each optimization run.
        :param weights: List of weights for the problems to adjust their mutual importance.
        :return: Object instance.
        """

        # Copy arguments to instance variables.
        self.optimizer = optimizer
        self.num_runs = num_runs
        self.max_evaluations = max_evaluations

        # Wrap the problems and weights. This is used for ranking the problems
        # which significantly speeds up the execution time, as explained below.
        if weights is None:
            # No weights were supplied so we just wrap the problems.
            self.problem_ranks = [_ProblemRank(problem) for problem in problems]
        else:
            # Wrap both the problems and weights.
            self.problem_ranks = [_ProblemRank(problem, weight) for problem, weight in zip(problems, weights)]

        # The MetaFitness-class is actually an optimization problem,
        # so init the parent-class.
        # The dimensionality of the search-space is the number of
        # control parameters for the optimizer, and the search-space
        # boundaries are the boundaries for the control parameters.
        Problem.__init__(self, name="MetaFitness",
                         dim=optimizer.num_parameters, fitness_min=0.0,
                         lower_bound=optimizer.parameters_lower_bound,
                         upper_bound=optimizer.parameters_upper_bound)

    def fitness(self, x, limit=np.Infinity):
        """
        Calculate the meta-fitness measure.

        :param x:
            Control parameters for the optimization method.

        :param limit:
            Abort the calculation of the meta-fitness when it
            becomes greater than this limit.

        :return:
            The meta-fitness measures how well the optimizer
            performed on the list of problems and using the given
            control parameters.
        """

        # Start a timer so we can later print the time-usage.
        timer = Timer()

        # Convenience variables.
        optimizer = self.optimizer
        max_evaluations = self.max_evaluations

        # Initialize the meta-fitness to zero.
        # The meta-fitness is just the (adjusted) sum of the
        # fitness obtained on multiple optimization runs.
        fitness_sum = 0.0

        # For each problem do the following.
        # Note that we iterate over self.problem_ranks which
        # is sorted so that we first try and optimize the problems
        # that are most likely to cause fitness_sum to exceed the
        # limit so the calculation can be aborted. This is called
        # Pre-Emptive Fitness Evaluation and greatly saves run-time.
        for problem_rank in self.problem_ranks:
            # Convenience variables.
            problem = problem_rank.problem
            weight = problem_rank.weight

            # Initialize the fitness sum for this problem.
            fitness_sum_inner = 0.0

            # Perform a number of optimization runs on the problem.
            for i in range(self.num_runs):
                # Perform one optimization run on the given problem
                # using the given control parameters.
                result = optimizer(problem=problem,
                                   max_evaluations=max_evaluations,
                                   parameters=x)

                # Keep track of the best-found solution for this problem.
                problem_rank.update_best(best=result.best,
                                         best_fitness=result.best_fitness)

                # Adjust the fitness so it is non-negative.
                fitness_adjusted = result.best_fitness - problem.fitness_min

                # Print warning if adjusted fitness is negative. Due to tiny rounding
                # errors this might occur without being an issue. But if the adjusted
                # fitness is negative and large, then problem.fitness_min must be corrected
                # in order for Pre-Emptive Fitness Evaluation to work properly.
                # It is better to print a warning than to use an assert which would
                # stop the execution.
                if fitness_adjusted < 0.0:
                    msg = "WARNING: MetaFitness.py, fitness_adjusted is negative {0:.4e} on problem {1}"
                    print(msg.format(fitness_adjusted, problem.name))

                # Accumulate the fitness sum for the inner-loop.
                fitness_sum_inner += weight * fitness_adjusted

                # Accumulate the overall fitness sum.
                fitness_sum += weight * fitness_adjusted

                # If the fitness sum exceeds the limit then break from the inner for-loop.
                if fitness_sum > limit:
                    break

            # Update the problem's ranking with the fitness-sum.
            # This is the key used in sorting below.
            problem_rank.fitness_sum = fitness_sum_inner

            # If the fitness sum exceeds the limit then break from the outer for-loop.
            if fitness_sum > limit:
                break

        # Sort the problems using the fitness_sum as the key in descending order.
        # This increases the probability that the for-loops above can be
        # aborted pre-emptively the next time the meta-fitness is calculated.
        self.problem_ranks = sorted(self.problem_ranks,
                                    key=lambda rank: rank.fitness_sum,
                                    reverse=True)

        # Stop the timer.
        timer.stop()

        # Print various results so we can follow the progress.
        print("- Parameters tried: {0}".format(x))
        msg = "- Meta-Fitness: {0:.4e}, Improvement: {1}"
        improvement = fitness_sum < limit
        print(msg.format(fitness_sum, improvement))
        print("- Time-Usage: {0}".format(timer))

        return fitness_sum


########################################################################

class MetaOptimize:
    """
    Performs meta-optimization, that is, tuning of an optimizer's
    control parameters by using another overlaying optimizer.
    """

    def __init__(self, optimizer, problems, num_runs, max_evaluations,
                 meta_num_runs=5, meta_max_evaluations=None, weights=None,
                 log_capacity=20, parallel=False):
        """
        Create object instance and perform the meta-optimization.

        :param optimizer:
            Optimizer-class, e.g. PSO, MOL or DE.

        :param problems:
            List of instances of the Problem-class.

        :param num_runs:
            Number of optimization runs to perform for each problem.

        :param max_evaluations:
            Number of fitness evaluations for each optimization run.

        :param meta_num_runs:
            Number of runs for the meta-optimizer (default 5).

        :param meta_max_evaluations:
            Number of iterations for each run of the meta-optimizer.
            If None then it is set to 20 * number of control parameters for the optimizer.

        :param weights:
            List of weights for the problems to adjust their mutual importance
            when tuning the control parameters of the optimizer.
            If weights=None then the weight is set to 1.0 for all problems.

        :param log_capacity:
            How many of the best control parameters are logged.

        :param parallel:
            Execute the meta-optimization runs in parallel. If True then only
            the best-found control parameters are available afterwards, as the
            log of best parameters will be empty. The best-found solutions
            to the optimization problems will also be empty.
            See README.md for more details.

        :return:
            Object instance. Get the optimization results from the object's variables.
            -   results.best are the best control parameters found for the optimizer.
            -   results.best_fitness is the meta-fitness of the best-found parameters.
            -   log.solutions holds several of the best control parameters.
            -   meta_fitness.problem_rank is a list which also holds the best-found
                solutions to the actual optimization problems.
        """

        # Default number of iterations for the meta-optimizer.
        if meta_max_evaluations is None:
            meta_max_evaluations = 20 * optimizer.num_parameters

        # Create a meta-fitness object using the optimizer, problems, weights, etc.
        self.meta_fitness = _MetaFitness(optimizer=optimizer,
                                         problems=problems,
                                         weights=weights,
                                         num_runs=num_runs,
                                         max_evaluations=max_evaluations)

        # Wrap the meta_fitness in a logger so we keep several of the best solutions.
        self.log = LogSolutions(problem=self.meta_fitness, capacity=log_capacity)

        # Perform multiple runs using LUS as the meta-optimizer which is
        # used because it usually requires very few iterations to find
        # near-optimal control parameters for the optimizer. However,
        # it typically needs about 5 runs because it can easily get
        # stuck with sub-optimal parameters.
        self.results = MultiRun(optimizer=LUS,
                                problem=self.log,  # Wrapper for self.meta_fitness
                                num_runs=meta_num_runs,
                                max_evaluations=meta_max_evaluations,
                                parallel=parallel,
                                display_interval=1, trace_len=100)


########################################################################



########################################################################
# SwarmOps - Heuristic optimization for Python.
# Copyright (C) 2003-2016 Magnus Erik Hvass Pedersen.
# See the file README.md for instructions.
# See the file LICENSE.txt for license details.
# SwarmOps on the internet: http://www.Hvass-Labs.org/
########################################################################

########################################################################
# Trace the fitness progress during optimization so it can be plotted.
########################################################################


class FitnessTrace:
    """
    Trace / log the fitness at regular intervals during optimization.
    This is used for plotting the optimization progress afterwards.
    """

    def __init__(self, trace_len, max_evaluations):
        """
        Create the object instance.

        :param trace_len: Max length of fitness-trace. Zero if no fitness-trace is wanted.
        :param max_evaluations: Number of optimization iterations that will be performed.
        :return: Object instance.
        """

        # The length of the fitness trace cannot be greater than
        # the number of optimization iterations.
        self.max_len = min(trace_len, max_evaluations)

        # Initialize empty arrays for the fitness-trace and iteration counter.
        # See performance note below.
        self.iteration = []         # Plot this as the x-axis.
        self.fitness = []           # Plot this as the y-axis.

        # Initialize the iteration counter used to decide when to trace/log the fitness.
        self.next_iteration = 0

        if self.max_len > 0:
            # The iteration interval between each trace of the fitness.
            self.interval = max_evaluations // self.max_len

    def trace(self, iteration, fitness):
        """
        Trace the fitness at regular intervals.

        :param iteration:  The number of optimization iterations performed so far.
        :param fitness: The best fitness found so far.
        :return: Nothing.
        """

        # Note:
        # Appending to a list is by far the most elegant way of implementing this.
        # The performance overhead of extending the list is tiny in comparison to
        # the total runtime of the optimization. Using a pre-allocated array instead
        # requires math with special cases and makes the implementation error-prone.

        # Proceed if a trace is desired.
        if self.max_len > 0:
            # Trace the fitness at regular intervals.
            if iteration >= self.next_iteration:
                # Append the fitness to the trace-array.
                self.fitness.append(fitness)

                # Append the iteration counter to the array.
                self.iteration.append(iteration)

                # Increment the counter for the next trace.
                self.next_iteration = iteration + self.interval

########################################################################



########################################################################
# SwarmOps - Heuristic optimization for Python.
# Copyright (C) 2003-2016 Magnus Erik Hvass Pedersen.
# See the file README.md for instructions.
# See the file LICENSE.txt for license details.
# SwarmOps on the internet: http://www.Hvass-Labs.org/
########################################################################

########################################################################
# Pattern Search (PS). An early variant was originally
# due to Fermi and Metropolis at the Los Alamos nuclear
# laboratory as described by Davidon [1]. It is also
# sometimes called compass search. This is a slightly
# different variant by Pedersen [2]. It works especially
# well when only few optimization iterations are allowed.
#
# References:
#
# [1] W.C. Davidon. Variable metric method for minimization.
#     SIAM Journal on Optimization, 1(1):1-17, 1991
#
# [2] M.E.H. Pedersen. Tuning & Simplifying Heuristical Optimization (PhD thesis).
#     University of Southampton, School of Engineering Sciences. 2010
#     http://www.hvass-labs.org/people/magnus/thesis/pedersen08thesis.pdf
#
########################################################################

from swarmops.Optimize import SingleRun
from swarmops import tools


########################################################################

class PS(SingleRun):
    """
        Perform a single optimization run using Pattern Search (PS).

        In practice, you would typically perform multiple optimization runs using
        the MultiRun-class. The reason is that PS is a heuristic optimizer so
        there is no guarantee that an acceptable solution is found in any single
        run. It is more likely that an acceptable solution is found if you perform
        multiple optimization runs.
    """

    # Name of this optimizer.
    name = "PS"
    name_full = "Pattern Search"

    @staticmethod
    def parameters_dict(parameters):
        """
        Create and return a dict from a list of PS parameters.
        The dict is empty because PS does not have any control parameters.
        This is implemented to have a consistent API.
        """

        return {}

    @staticmethod
    def parameters_list():
        """
        Create an empty list because PS does not have any parameters.
        This is implemented to have a consistent API.
        """

        return []

    def __init__(self, problem, parameters=None, parallel=False, *args, **kwargs):
        """
        Create object instance and perform a single optimization run using PS.

        :param problem: The problem to be optimized. Instance of Problem-class.

        :param parameters: None. There are no control parameters for PS.

        :param parallel: False. PS cannot run in parallel except through MultiRun.

        :return:
            Object instance. Get the optimization results from the object's variables.
            -   best is the best-found solution.
            -   best_fitness is the associated fitness of the best-found solution.
            -   fitness_trace is an instance of the FitnessTrace-class.
        """

        # Copy arguments to instance variables.
        self.problem = problem

        # Initialize parent-class which also starts the optimization run.
        SingleRun.__init__(self, *args, **kwargs)

    def _optimize(self):
        """
        Perform a single optimization run.
        This function is called by the parent-class.
        """

        # Convenience variable for fitness function.
        f = self.problem.fitness

        # Convenience variables for search-space boundaries.
        lower_init = self.problem.lower_init
        upper_init = self.problem.upper_init
        lower_bound = self.problem.lower_bound
        upper_bound = self.problem.upper_bound

        # Dimensionality of search-space.
        dim = self.problem.dim

        # Initialize the range-vector to full search-space.
        d = upper_bound - lower_bound

        # Initialize x with random position in search-space.
        x = tools.rand_array(lower=lower_init, upper=upper_init)

        # Compute fitness of initial position.
        fitness = f(x)

        # Update the best-known fitness and position.
        # The parent-class is used for this.
        self._update_best(fitness=fitness, x=x)

        # Perform optimization iterations until the maximum number
        # of fitness evaluations has been performed.
        # Count starts at one because we have already calculated fitness once above.
        evaluations = 1
        while evaluations < self.max_evaluations:
            # Pick random index from {0, .., dim-1}
            idx = tools.rand_int(lower=0, upper=dim)

            # Save old element.
            t = x[idx]

            # Make new element.
            x[idx] = x[idx] + d[idx]

            # Bound new element to search-space.
            x[idx] = tools.bound_scalar(x=x[idx],
                                        lower=lower_bound[idx],
                                        upper=upper_bound[idx])

            # Compute new fitness.
            new_fitness = f(x, limit=fitness)

            # If improvement to fitness.
            if new_fitness < fitness:
                # Update just the fitness as the position is already updated.
                fitness = new_fitness

                # Update the best-known fitness and position.
                # The parent-class is used for this.
                self._update_best(fitness=fitness, x=x)
            else:
                # Otherwise restore the position.
                x[idx] = t

                # Reduce search-range and invert direction for this dimension.
                d[idx] *= -0.5

            # Call parent-class to print status etc. during optimization.
            self._iteration(evaluations)

            # Increment counter.
            evaluations += 1

########################################################################



########################################################################
# SwarmOps - Heuristic optimization for Python.
# Copyright (C) 2003-2016 Magnus Erik Hvass Pedersen.
# See the file README.md for instructions.
# See the file LICENSE.txt for license details.
# SwarmOps on the internet: http://www.Hvass-Labs.org/
########################################################################

########################################################################
# Timer-class used for timing execution.
########################################################################

import time
from datetime import timedelta


########################################################################

class Timer:
    """
    Used for timing execution.
    """

    def __init__(self):
        """
        Start the timer.

        :return: Object instance.
        """

        # Note that time.process_time() doesn't work with multiprocessing.

        self.start_time = time.time()
        self.end_time = self.start_time

    def less_than(self, hours=0, minutes=0, seconds=0):
        """
        Return True if the time that has passed between now and the timer was
        started is less than the given hours, minutes and seconds,
        otherwise return False.

        Note that this does not stop the timer.
        """

        # The current time.
        now = time.time()

        # Time-difference between now and the timer was started.
        time_dif = now - self.start_time

        # Time-difference as a time-delta.
        delta = timedelta(seconds=time_dif)

        # Limit as a time-delta.
        limit_delta = timedelta(hours=hours, minutes=minutes, seconds=seconds)

        # Compare the two time-deltas.
        return delta < limit_delta

    def stop(self):
        """
        Stop the timer.
        """

        self.end_time = time.time()

    def __str__(self):
        """
        The difference between start and end-time is converted to a string.
        """

        time_dif = self.end_time - self.start_time
        return str(timedelta(seconds=int(round(time_dif))))

########################################################################



########################################################################
# SwarmOps - Heuristic optimization for Python.
# Copyright (C) 2003-2016 Magnus Erik Hvass Pedersen.
# See the file README.md for instructions.
# See the file LICENSE.txt for license details.
# SwarmOps on the internet: http://www.Hvass-Labs.org/
########################################################################

########################################################################
# Various tools for random numbers, boundaries, etc.
########################################################################

import numpy as np


########################################################################
# Pseudo-Random Number Generator (PRNG).

# The PRNG used to generate random numbers for an execution thread.
# This is not thread-safe so a new PRNG must be created for each thread
# using the new_prng() function below.
_prng = np.random.RandomState()


def new_prng(seed=None):
    """
    Create a new PRNG. Call this function when you start a new execution thread.
    This is necessary because the PRNG is not thread-safe so each thread must have
    its own PRNG object.

    :param seed: Starting seed for the PRNG.
    :return: Nothing.
    """

    # This ensures that we update the module's variable instead of
    # merely creating a new local variable inside this function.
    global _prng

    # Create a new PRNG object for the module.
    _prng = np.random.RandomState(seed=seed)


def rand_uniform(size):
    """
    Creates an array of the given size with random uniform numbers between [0, 1).

    Wraps the rand() function from numpy.
    """

    return _prng.rand(size)


def rand_array(lower, upper):
    """
    Create array with uniform random numbers between the given lower and upper bounds.
    """

    return (upper - lower) * _prng.rand(*lower.shape) + lower


def rand_population(lower, upper, num_agents, dim):
    """
    Create 2-d array with uniform random numbers between the lower and upper bounds.

    The first index is for the agent numbers.
    The second index is for the search-space dimensionality.

    :param lower: Lower boundary.
    :param upper: Upper boundary.
    :param num_agents: Number of agents in the population.
    :param dim: Dimensionality of the search-space.
    :return: 2-d array with uniform random values between bounds.
    """

    return (upper - lower) * _prng.rand(num_agents, dim) + lower


def rand_int(lower, upper):
    """
    Return a single random integer between lower (inclusive) and upper (exclusive).

    Wraps the randint() function from numpy.
    """

    return _prng.randint(low=lower, high=upper)


def rand_choice(a, size=None, replace=True, p=None):
    """
    Generate a random sample from an array.

    Wraps the choice() function from numpy so the parameters are the same.

    :param a:
        If a numpy ndarray, then a random sample is generated from its elements.
        If an int, then the random sample is generated as if a was np.arange(a)

    :param size:
        Output shape. Int or tuple of ints.

    :param replace:
        Sample with or without replacement.

    :param p:
        The probabilities associated with each entry in a.
        If not given then the sample assumes a uniform distribution over
        all entries in a.

    :return:
        Array with random sample.
    """

    return _prng.choice(a=a, size=size, replace=replace, p=p)


def sample_bounded(x, d, lower, upper):
    """
    Generate a random sample between x-d and x+d, while ensuring the range
    is bounded by lower and upper.
    """

    # Adjust sampling range so it does not exceed the search-space boundaries.
    l = np.maximum(x - d, lower)
    u = np.minimum(x + d, upper)

    # Return a random sample.
    return rand_array(lower=l, upper=u)


########################################################################

def bound(x, lower, upper):
    """
    Bound x between lower and upper, where x is a numpy array.
    """

    # Lower bound.
    y = np.where(x < lower, lower, x)

    # Upper bound.
    z = np.where(y > upper, upper, y)

    return z


def bound_scalar(x, lower, upper):
    """
    Bound x between lower and upper, where x is a scalar value.
    """

    return min(upper, max(lower, x))

########################################################################

def denormalize_trunc(x):
    """
    If x is very close to zero then it is truncated to zero.
    This is done to avoid denormalized floating point values.

    It is unclear from the Python and numpy manuals whether this is a problem, but
    in other languages (e.g. C# and C++) arithmetic operations can perform extremely slowly
    on denormalized floats. Some choices of PSO parameters cause this problem to occur.

    The truncating is very quick and only has a tiny runtime penalty.
    """

    # Truncate values to zero if they are below this limit.
    limit = 1e-30

    return np.where(np.abs(x) > limit, x, 0.0)

########################################################################



########################################################################
# SwarmOps - Heuristic optimization for Python.
# Copyright (C) 2003-2016 Magnus Erik Hvass Pedersen.
# See the file README.md for instructions.
# See the file LICENSE.txt for license details.
# SwarmOps on the internet: http://www.Hvass-Labs.org/
########################################################################

########################################################################
# Provides logging of best solutions found for an optimization problem,
# so we get a list of good solutions rather than just a single solution.
########################################################################

import numpy as np


########################################################################

class _LogElement:
    """
    Used for storing parameters and associated fitness in the log.
    """

    def __init__(self, x=None, fitness=np.Infinity):
        """
        Create object instance.

        :param x: Position in the search-space aka. candidate solution.
        :param fitness: Associated fitness of the position in the search-space.
        :return: Object instance.
        """

        # Copy arguments to instance variables.
        self.x = x
        self.fitness = fitness


########################################################################

class LogSolutions:
    """
    Transparently wraps a Problem-object and provides logging of the
    best solutions found. This allows us to get a list of good
    solutions for a problem rather than just the single best solution.
    """

    def __init__(self, problem, capacity=20):
        """
        Create object instance.

        :param problem: Instance of the Problem-class to be wrapped.
        :param capacity: Capacity of the log, default 20.
        :return: Object instance.
        """

        # Copy all the attributes of the problem-object to self. This essentially
        # wraps the problem-object and makes e.g. self.dim the same as problem.dim, etc.
        # Note that a shallow copy is being made, otherwise self.__dict__
        # would be a reference to problem.__dict__ which would cause the
        # original problem-object to become modified e.g. with self.capacity, etc.
        self.__dict__ = problem.__dict__.copy()

        # Copy arguments to instance variables.
        self.problem = problem
        self.capacity = capacity

        # Initialize log with empty solutions.
        self.solutions = [_LogElement() for i in range(capacity)]

    def fitness(self, x, limit=np.Infinity):
        """
        Wraps the fitness-function of the actual problem, whose results
        are logged if the fitness is an improvement.
        """

        # Calculate fitness of the actual problem.
        new_fitness = self.problem.fitness(x=x, limit=limit)

        # If log is desired.
        if self.capacity > 0:
            # If the new fitness is an improvement over the worst-known fitness in the log.
            if new_fitness < self.solutions[-1].fitness:
                # Update the worst solution in the log with the new solution.
                self.solutions[-1] = _LogElement(x=x, fitness=new_fitness)

            # Sort the logged solutions.
            self.solutions = sorted(self.solutions, key=lambda solution: solution.fitness)

        return new_fitness

########################################################################



# -*- coding: utf-8 -*-
from setuptools import setup

MY_VERSION = '1.0.0'

setup(
    name='swarmops',
    packages=['swarmops'],
    version=MY_VERSION,
    description='Heuristic Optimization for Python',
    author='Magnus Erik Hvass Pedersen',
    author_email='my-first-name@hvass-labs.org',
    url='https://github.com/Hvass-Labs/swarmops',
#    download_url='git+https://github.com/Hvass-Labs/swarmops.git',
    license='BSD',
    keywords=['heuristic optimization', 'particle swarm optimization',
              'differential evolution', 'meta-optimization'],
    install_requires=[
        'numpy>=1.9',
        'scipy>=0.16',
        'matplotlib>=1.5',
    ],
)
